package ffmpegconverter.converter;

import java.io.File;
import java.io.IOException;
import java.util.List;

public class VideoConverter extends Converter {
    private String newExt;

    public VideoConverter(String ffprobePath, String ffmpegPath, String newExt) {
        this.ffprobePath = ffprobePath;
        this.ffmpegPath = ffmpegPath;
        this.newExt = newExt;
    }

    /**
     * Reads streams from a file, and converts it to an mp4.
     *
     * @param folder        The folder of the file to process
     * @param file          The file to process
     * @throws IOException  If the BufferedReader fails
     */
    private void processFile(File folder, File file, String newExt) throws IOException {
        String[] streams = probeFile(ffprobePath, file);
        if (streams.length == 0) {
            throw new IllegalArgumentException("The file has no streams");
        }
        String newPath = stripExtension(file) + Math.random() + "." + newExt;
        convertProcess(new ProcessBuilder(builderCommand(ffmpegPath, file.getName(), streams, newPath)), folder);
    }

    /**
     * Generates a command for a ProcessBuilder.
     *
     * @param executable    The executable file for ffmpeg
     * @param fileName      The input file
     * @param streams       A list of ffprobe streams
     * @param outFile       The output file
     * @return              A list of commands
     */
    private String[] builderCommand(String executable, String fileName, String[] streams, String outFile) {
        List<String> command = generalFile(executable, fileName);
        List<Integer> videoStreams = listVideo(streams);
        if (videoStreams.size() > 0) {
            command.add("-map");
            command.add("0:" + videoStreams.get(0));
        }
        List<Integer> audioStreams = listAudio(streams);
        if (audioStreams.size() > 0) {
            command.add("-map");
            command.add("0:" + audioStreams.get(0));
        }
        command.add(outFile);
        return command.toArray(new String[0]);
    }

    @Override
    public String[] getValidFormats() {
        return VIDEO_FORMATS;
    }

    @Override
    public void convert(File file) throws IOException {
        processFile(file.getParentFile(), file, newExt);
    }
}